@@ -4,6 +4,7 @@
 #include <sys/time.h>
 
 #include "arch/arch.h"
+#include "lib/seqlock.h"
 
 /*
  * Clock sources
@@ -15,6 +16,7 @@ enum fio_cs {
 	CS_INVAL,
 };
 
+extern int fio_get_mono_time(struct timespec *);
 extern void fio_gettime(struct timespec *, void *);
 extern void fio_gtod_init(void);
 extern void fio_clock_init(void);
@@ -22,20 +24,22 @@ extern int fio_start_gtod_thread(void);
 extern int fio_monotonic_clocktest(int debug);
 extern void fio_local_clock_init(void);
 
-extern struct timespec *fio_ts;
+extern struct fio_ts {
+	struct seqlock seqlock;
+	struct timespec ts;
+} *fio_ts;
 
 static inline int fio_gettime_offload(struct timespec *ts)
 {
-	time_t last_sec;
+	unsigned int seq;
 
 	if (!fio_ts)
 		return 0;
 
 	do {
-		read_barrier();
-		last_sec = ts->tv_sec = fio_ts->tv_sec;
-		ts->tv_nsec = fio_ts->tv_nsec;
-	} while (fio_ts->tv_sec != last_sec);
+		seq = read_seqlock_begin(&fio_ts->seqlock);
+		*ts = fio_ts->ts;
+	} while (read_seqlock_retry(&fio_ts->seqlock, seq));
 
 	return 1;
 }
